#!/usr/bin/perl

# Copyright (C) 2005 John J. Chew, III <jjchew@math.utoronto.ca>
# All Rights Reserved

package TSH::Command::Addscore;

use strict;
use warnings;

use TSH::Utility;
use TSH::Tournament;

our (@ISA) = qw(TSH::Command);

=pod

=head1 NAME

TSH::Command::Addscore - implement the C<tsh> Addscore command

=head1 SYNOPSIS

  my $command = new TSH::Command::Addscore;
  my $argsp = $command->ArgumentTypes();
  my $helptext = $command->Help();
  my (@names) = $command->Names();
  $command->Run($tournament, @parsed_arguments);
  
=head1 ABSTRACT

TSH::Command::Addscore is a subclass of TSH::Command.

=cut

=head1 DESCRIPTION

=over 4

=cut

sub initialise ($$$$);
sub new ($);
sub Run ($$@);

=item $parserp->initialise()

Used internally to (re)initialise the object.

=cut

sub initialise ($$$$) {
  my $this = shift;
  my $path = shift;
  my $namesp = shift;
  my $argtypesp = shift;

  $this->{'help'} = <<'EOF';
Use this command to enter player scores.  You must pair the round
(e.g. by autopairing or using the pm command) before you can enter
any scores.  Begin by specifying the round and division that you
are entering.  At the prompt, enter the first player's number and
score, then the second player's number and score, all on one line.
For a bye or forfeit, enter the player's number and the spread
adjustment.  You may enter a division name to switch divisions, or
'm' to see what scores are still missing.  To correct a mistake in
the game you just entered, enter 'es'.  If you enter anything else,
you will exit the command and return to the main prompt.
EOF
  $this->{'names'} = [qw(a addscore)];
  $this->{'argtypes'} = [qw(Round Division)];
# print "names=@$namesp argtypes=@$argtypesp\n";

  return $this;
  }

sub new ($) { return TSH::Utility::new(@_); }

=item $command->Run($tournament, @parsed_args)

Should run the command in the context of the given
tournament with the specified parsed arguments.

=cut

# TODO: split this up into smaller subs for maintainability

sub Run ($$@) { 
  my $this = shift;
  my $tournament = shift;
  my ($round, $dp) = @_;
  my $round0 = $round - 1;
  if ($round0 <= $dp->{'mins'}) {
    $tournament->TellUser('eallsin', $dp->Name(), $round);
    return;
    }
  if ($round0 > $dp->{'mins'}+1) {
    $tournament->TellUser('emisss', $dp->Name(),
      $dp->{'mins'}+2, $dp->LeastScoresPlayer()->TaggedName());
    return;
    }
  my $datap = $dp->{'data'};

  my $lastpn1 = 1;
  my $changed = 0;
  while (1) {
    my $left = scalar(grep { (!exists $_->{'etc'}{'off'}) && ! defined $_->{'scores'}[$round0] } 
      @$datap[1..$#$datap]);
    $left = $left == 1 ? "$left score" : "$left scores";
    # TODO: write and use TSH::Utility::Prompt
    TSH::Utility::Prompt 
      "[$dp->{'name'}${round}]:pn1 ps1 pn2 ps2 ($left left)?";
    local($_) = scalar(<STDIN>);
    s/\s+$//;
    if (my $newdp = $tournament->GetDivisionByName($_)) {
      $dp = $newdp;
      $datap = $dp->{'data'};
      last unless defined $datap;
      if ($round0 <= $newdp->{'mins'}) {
	$tournament->TellUser('eallsin', $newdp->Name(), $round);
	return;
	}
      if ($round0 > $newdp->{'mins'}+1) {
	$tournament->TellUser('emisss', $newdp->Name(),
	  $dp->{'mins'}+1, $newdp->LeastScoresPlayer()->TaggedName());
	return;
	}
      next;
      }
    elsif (/^(?:m|miss|missing)$/i) {
      $tournament->RunCommand('MISSing', $round);
      next;
      }
    elsif (/^(?:es|editscore)$/i) {
      $tournament->UpdateDivisions();
      $tournament->RunCommand('EditScore', $dp->{'name'}, $lastpn1, $round);
      next;
      }
    last if /[^-\d\s]/;
    my (@words) = split;
    if (@words == 2) {
      my ($pn1, $ps1) = @words;
      if ($pn1 < 1 || $pn1 > $#$datap) { 
	$tournament->TellUser('enosuchp', $pn1);
	next;
	}
      my $pp1 = $datap->[$pn1];
      my $opp1 = $pp1->{'pairings'}[$round-1];
      unless ((defined $opp1) && $opp1 == 0) {
	$tournament->TellUser('enotabye', $pp1->TaggedName(), $round);
	next;
        }
      if ($pp1->{'scores'}[$round-1] && $pp1->{'scores'}[$round-1] != 9999) {
	$tournament->TellUser('ehass', $pp1->TaggedName(),
	  $pp1->Score($round-1));
	next;
	}
      {
	my $wlt = (($ps1 <=> 0) + 1) / 2;
	printf "#%d %s %+d (%.1f %+d).\n",
	  $pp1->{id}, 
	  $pp1->{name}, 
	  $ps1,
	  $pp1->{wins} + $wlt,
	  $pp1->{spread} + $ps1,
	  ;
      }
      $lastpn1 = $pn1;
      $dp->Dirty(1);
      $changed++;
      $pp1->{'etc'}{'time'} = [time];
      $pp1->{'scores'}[$round-1] = $ps1;
      next;
      }
    last unless @words == 4;
    my ($pn1, $ps1, $pn2, $ps2) = @words;
    if ($pn1 < 1 || $pn1 > $#$datap) {
      $tournament->TellUser('enosuchp', $pn1);
      next;
      }
    if ($pn2 < 1 || $pn2 > $#$datap) {
      $tournament->TellUser('enosuchp', $pn2);
      next;
      }
    if ($ps1 < -150 || $ps1 > 1500) {
      $tournament->TellUser('ebadscore', $ps1);
      next;
      }
    if ($ps2 < -150 || $ps2 > 1500) {
      $tournament->TellUser('ebadscore', $ps2);
      next;
      }
    my $pp1 = $datap->[$pn1];
    my $pp2 = $datap->[$pn2];
    if ($pp1->{'pairings'}[$round-1] ne $pn2) {
      TSH::Utility::Error "$pp1->{'name'} and $pp2->{'name'} did not play each other in round $round.\n";
      next;
      }
    if ((defined $pp1->Score($round-1)) && $pp1->Score($round-1) != 9999) {
      $tournament->TellUser('ehass', $pp1->TaggedName(),
	$pp1->Score($round-1));
      next;
      }
    if ((defined $pp2->Score($round-1)) && $pp2->Score($round-2) != 9999) {
      $tournament->TellUser('ehass', $pp2->TaggedName(),
	$pp2->Score($round-1));
      next;   
      }
    if ($config::track_firsts && !$config::assign_firsts) {
      my $p12p = $pp1->{'etc'}{'p12'};
      my $old = $p12p->[$round-1];
      if ($old && $old == 2) 
        { $tournament->TellUser('easbad12', $pp1->TaggedName(), 'second'); }
      $p12p->[$round-1] = 1;

      $p12p = $pp2->{'etc'}{'p12'};
      $old = $p12p->[$round-1];
      if ($old && $old == 1) 
        { $tournament->TellUser('easbad12', $pp2->TaggedName(), 'first'); }
      $p12p->[$round-1] = 2;
      }
    {
      my $spread = $ps1 - $ps2;
      my $wlt = (($spread <=> 0) + 1) / 2;
      printf "#%d %s %d (%.1f %+d) - #%d %s %d (%.1f %+d).\n",
        $pp1->{id}, 
        $pp1->{name}, 
	$ps1,
	$pp1->{wins} + $wlt,
	$pp1->{spread} + $spread,
        $pp2->{id}, 
        $pp2->{name}, 
	$ps2,
	$pp2->{wins} + 1 - $wlt,
	$pp2->{spread} - $spread,
	;
    }
    $lastpn1 = $pn1;
    $dp->Dirty(1);
    $changed++;
    $pp1->{'etc'}{'time'} = $pp2->{'etc'}{'time'} = [time];
    $pp1->{'scores'}[$round-1] = $ps1;
    $pp2->{'scores'}[$round-1] = $ps2;
    }
  continue {
    if ($changed >= ($config'save_interval||10)) {
      $tournament->UpdateDivisions();
      $changed = 0;
      }
    }
  $tournament->UpdateDivisions();
  }

=back

=cut

=head1 BUGS

None known.

=cut

1;
